/*!
 * @file        bsp_cec.h
 *
 * @brief       Header for bsp_cec.c module
 *
 * @version     V1.0.0
 *
 * @date        2022-09-29
 *
 * @attention
 *
 *  Copyright (C) 2022 Geehy Semiconductor
 *
 *  You may not use this file except in compliance with the
 *  GEEHY COPYRIGHT NOTICE (GEEHY SOFTWARE PACKAGE LICENSE).
 *
 *  The program is only for reference, which is distributed in the hope
 *  that it will be usefull and instructional for customers to develop
 *  their software. Unless required by applicable law or agreed to in
 *  writing, the program is distributed on an "AS IS" BASIS, WITHOUT
 *  ANY WARRANTY OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the GEEHY SOFTWARE PACKAGE LICENSE for the governing permissions
 *  and limitations under the License.
 */

/* Define to prevent recursive inclusion */
#ifndef __BSP_CEC_H
#define __BSP_CEC_H

/* Includes */
#include "main.h"

/** @addtogroup Board
  @{
*/

/** @addtogroup Board_APM32F072_EVAL
  @{
*/

/** @defgroup APM32F072_EVAL_Macros Macros
  @{
*/

/* 0x04 for CEC version: V1.3.a
   0x05 for CEC version: V1.4
*/
#define CEC_VERSION                                     0x05

/**
  * @brief  CEC Root CFG (Mainly for TV with a fixed physical address (0.0.0.0))
  *         If you want to configure the APM32F072B_EVAL board as CEC Root (Sink)
  *         change the following define to 0x01
  */
#define CEC_ROOT_CFG                                    0x00

#define CEC_DEVICE_MAX                                  13
#define CEC_DEV_COMMAND_MAX                             10
#define CEC_TX_MSG_LEN_MAX                              ((uint32_t)0x0000000E)
#define CEC_TIMEOUT_VAL                                 ((uint32_t)0x001FFFFF)

#define CEC_SOURCE_I2C_SCL_PIN                          GPIO_PIN_6
#define CEC_SOURCE_I2C_SDA_PIN                          GPIO_PIN_7
#define CEC_SOURCE_I2C_SCL_PIN_SOURCE                   GPIO_PIN_SOURCE_6
#define CEC_SOURCE_I2C_SDA_PIN_SOURCE                   GPIO_PIN_SOURCE_7
#define CEC_SOURCE_I2C_GPIO_PORT                        GPIOB
#define CEC_SOURCE_I2C_GPIO_CLK                         RCM_AHB_PERIPH_GPIOB
#define CEC_SOURCE_I2C                                  I2C1
#define CEC_SOURCE_I2C_CLK                              RCM_APB1_PERIPH_I2C1

#define CEC_SINK_I2C_SCL_PIN                            GPIO_PIN_13
#define CEC_SINK_I2C_SDA_PIN                            GPIO_PIN_14
#define CEC_SINK_I2C_SCL_PIN_SOURCE                     GPIO_PIN_SOURCE_13
#define CEC_SINK_I2C_SDA_PIN_SOURCE                     GPIO_PIN_SOURCE_14
#define CEC_SINK_I2C_GPIO_PORT                          GPIOB
#define CEC_SINK_I2C_GPIO_CLK                           RCC_AHBPeriph_GPIOB
#define CEC_SINK_I2C                                    I2C2
#define CEC_SINK_I2C_CLK                                RCM_APB1_PERIPH_I2C2

#define CEC_LINE_PIN                                    GPIO_PIN_8
#define CEC_LINE_GPIO_PORT                              GPIOB
#define CEC_LINE_SOURCE                                 GPIO_PIN_SOURCE_8
#define CEC_LINE_GPIO_CLK                               RCM_AHB_PERIPH_GPIOB
#define CEC_LINE_GPIO_AF                                GPIO_AF_PIN0

#define CEC_HPD_SOURCE_PIN                              GPIO_PIN_0
#define CEC_HPD_SOURCE_GPIO_PORT                        GPIOE
#define CEC_HPD_SOURCE_GPIO_CLK                         RCM_AHB_PERIPH_GPIOE

#define CEC_HPD_SINK_PIN                                GPIO_PIN_15
#define CEC_HPD_SINK_GPIO_PORT                          GPIOD
#define CEC_HPD_SINK_GPIO_CLK                           RCM_AHB_PERIPH_GPIOD

/* Reason for Abort feature*/
#define CEC_UNRECOGNIZED_OPCODE                         0x00
#define CEC_NOT_CORRECT_MODETORESPOND                   0x01
#define CEC_CANNOTPROVIDE_SOURCE                        0x02
#define CEC_INVALID_OPERAND                             0x03
#define CEC_REFUSED                                     0x04

/* HDMI CEC Devices connected status*/
#define CEC_CONNECT_TV                                  (uint16_t)0x0001
#define CEC_CONNECT_RECORDING1                          (uint16_t)0x0002
#define CEC_CONNECT_RECORDING2                          (uint16_t)0x0004
#define CEC_CONNECT_TUNER1                              (uint16_t)0x0008
#define CEC_CONNECT_PLAYBACK1                           (uint16_t)0x0010
#define CEC_CONNECT_AUDIO                               (uint16_t)0x0020
#define CEC_CONNECT_TUNER2                              (uint16_t)0x0040
#define CEC_CONNECT_TUNER3                              (uint16_t)0x0080
#define CEC_CONNECT_PLAYBACK2                           (uint16_t)0x0100
#define CEC_CONNECT_RECORDING3                          (uint16_t)0x0200
#define CEC_CONNECT_TUNER4                              (uint16_t)0x0400
#define CEC_CONNECT_PLAYBACK3                           (uint16_t)0x0800
#define CEC_CONNECT_BROADCAST                           (uint16_t)0x1000

/* HDMI CEC Opcode*/
#define CEC_FEATURE_ABORT_OPCODE                        ((uint8_t) 0x00)
#define CEC_IMAGE_VIEW_ON_OPCODE                        ((uint8_t) 0x04)
#define CEC_TUNER_STEP_INCREMENT_OPCODE                 ((uint8_t) 0x05)
#define CEC_TUNER_STEP_DECREMENT_OPCODE                 ((uint8_t) 0x06)
#define CEC_TUNER_DEVICE_STATUS_OPCODE                  ((uint8_t) 0x07)
#define CEC_GIVE_TUNER_DEVICE_STATUS_OPCODE             ((uint8_t) 0x08)
#define CEC_RECORD_ON_OPCODE                            ((uint8_t) 0x09)
#define CEC_RECORD_STATUS_OPCODE                        ((uint8_t) 0x0A)
#define CEC_RECORD_OFF_OPCODE                           ((uint8_t) 0x0B)
#define CEC_TEXT_VIEW_ON_OPCODE                         ((uint8_t) 0x0D)
#define CEC_RECORD_TV_SCREEN_OPCODE                     ((uint8_t) 0x0F)
#define CEC_GIVE_DECK_STATUS_OPCODE                     ((uint8_t) 0x1A)
#define CEC_DECK_STATUS_OPCODE                          ((uint8_t) 0x1B)
#define CEC_SET_MENU_LANGUAGE_OPCODE                    ((uint8_t) 0x32)
#define CEC_CLEAR_ANALOGUE_TIMER_OPCODE                 ((uint8_t) 0x33)
#define CEC_SET_ANALOGUE_TIMER_OPCODE                   ((uint8_t) 0x34)
#define CEC_TIMER_STATUS_OPCODE                         ((uint8_t) 0x35)
#define CEC_STANDBY_OPCODE                              ((uint8_t) 0x36)
#define CEC_PLAY_OPCODE                                 ((uint8_t) 0x41)
#define CEC_DECK_CONTROL_OPCODE                         ((uint8_t) 0x42)
#define CEC_TIMER_CLEARED_STATUS_OPCODE                 ((uint8_t) 0x43)
#define CEC_USER_CONTROL_PRESSED_OPCODE                 ((uint8_t) 0x44)
#define CEC_USER_CONTROL_RELEASED_OPCODE                ((uint8_t) 0x45)
#define CEC_GIVE_OSD_NAME_OPCODE                        ((uint8_t) 0x46)
#define CEC_SET_OSD_NAME_OPCODE                         ((uint8_t) 0x47)
#define CEC_SET_OSD_STRING_OPCODE                       ((uint8_t) 0x64)
#define CEC_SET_TIMER_PROGRAM_TITLE_OPCODE              ((uint8_t) 0x67)
#define CEC_SYSTEM_AUDIO_MODE_REQUEST_OPCODE            ((uint8_t) 0x70)
#define CEC_GIVE_AUDIO_STATUS_OPCODE                    ((uint8_t) 0x71)
#define CEC_SET_SYSTEM_AUDIO_MODE_OPCODE                ((uint8_t) 0x72)
#define CEC_REPORT_AUDIO_STATUS_OPCODE                  ((uint8_t) 0x7A)
#define CEC_GIVE_SYSTEM_AUDIO_MODE_STATUS_OPCODE        ((uint8_t) 0x7D)
#define CEC_SYSTEM_AUDIO_MODE_STATUS_OPCODE             ((uint8_t) 0x7E)
#define CEC_ROUTING_CHANGE_OPCODE                       ((uint8_t) 0x80)
#define CEC_ROUTING_INFORMATION_OPCODE                  ((uint8_t) 0x81)
#define CEC_ACTIVE_SOURCE_OPCODE                        ((uint8_t) 0x82)
#define CEC_GIVE_PHYSICAL_ADDRESS_OPCODE                ((uint8_t) 0x83)
#define CEC_REPORT_PHYSICAL_ADDRESS_OPCODE              ((uint8_t) 0x84)
#define CEC_REQUEST_ACTIVE_SOURCE_OPCODE                ((uint8_t) 0x85)
#define CEC_SET_STREAM_PATH_OPCODE                      ((uint8_t) 0x86)
#define CEC_DEVICE_VENDOR_ID_OPCODE                     ((uint8_t) 0x87)
#define CEC_VENDOR_COMMAND_OPCODE                       ((uint8_t) 0x89)
#define CEC_VENDOR_REMOTE_BUTTON_DOWN_OPCODE            ((uint8_t) 0x8A)
#define CEC_VENDOR_REMOTE_BUTTON_UP_OPCODE              ((uint8_t) 0x8B)
#define CEC_GIVE_DEVICE_VENDOR_ID_OPCODE                ((uint8_t) 0x8C)
#define CEC_MENU_REQUEST_OPCODE                         ((uint8_t) 0x8D)
#define CEC_MENU_STATUS_OPCODE                          ((uint8_t) 0x8E)
#define CEC_GIVE_DEVICE_POWER_STATUS_OPCODE             ((uint8_t) 0x8F)
#define CEC_REPORT_POWER_STATUS_OPCODE                  ((uint8_t) 0x90)
#define CEC_GET_MENU_LANGUAGE_OPCODE                    ((uint8_t) 0x91)
#define CEC_SELECT_ANALOGUE_SERVICE_OPCODE              ((uint8_t) 0x92)
#define CEC_SELECT_DIGITAL_SERVICE_OPCODE               ((uint8_t) 0x93)
#define CEC_SET_DIGITAL_TIMER_OPCODE                    ((uint8_t) 0x97)
#define CEC_CLEAR_DIGITAL_TIMER_OPCODE                  ((uint8_t) 0x99)
#define CEC_SET_AUDIO_RATE_OPCODE                       ((uint8_t) 0x9A)
#define CEC_INACTIVE_SOURCE_OPCODE                      ((uint8_t) 0x9D)
#define CEC_CEC_VERSION_OPCODE                          ((uint8_t) 0x9E)
#define CEC_GET_CEC_VERSION_OPCODE                      ((uint8_t) 0x9F)
#define CEC_VENDOR_COMMAND_WITH_ID_OPCODE               ((uint8_t) 0xA0)
#define CEC_CLEAR_EXTERNAL_TIMER_OPCODE                 ((uint8_t) 0xA1)
#define CEC_SET_EXTERNAL_TIMER_OPCODE                   ((uint8_t) 0xA2)
#define CEC_ABORT_OPCODE                                ((uint8_t) 0xFF)

/* HDMI CEC Enumerate*/
/**@} end of group APM32F072_EVAL_Macros*/

/** @defgroup APM32F072_EVAL_Enumerates Enumerates
  @{
  */

/**
 * @brief   CEC user control operation
 */
typedef enum
{
    USR_OP_SELECT,
    USR_OP_UP,
    USR_OP_DOWN,
    USR_OP_LEFT,
    USR_OP_RIGHT,
    USR_OP_RIGHT_UP,
    USR_OP_RIGHT_DOWN,
    USR_OP_LEFT_UP,
    USR_OP_LEFT_DOWN,
    USR_OP_ROOT_MENU,
    USR_OP_SETUP_MENU,
    USR_OP_CONTENTS_MENU,
    USR_OP_FAVORITE_MENU,
    USR_OP_EXIT,
    USR_OP_NUMBER_0 = 0x20,
    USR_OP_NUMBER_1,
    USR_OP_NUMBER_2,
    USR_OP_NUMBER_3,
    USR_OP_NUMBER_4,
    USR_OP_NUMBER_5,
    USR_OP_NUMBER_6,
    USR_OP_NUMBER_7,
    USR_OP_NUMBER_8,
    USR_OP_NUMBER_9,
    USR_OP_DOT,
    USR_OP_ENTER,
    USR_OP_CLEAR,
    USR_OP_CHANNEL_UP = 0x30,
    USR_OP_CHANNEL_DOWN,
    USR_OP_PREVIOUS_CHANNEL,
    USR_OP_SOUND_SELECT,
    USR_OP_INPUT_SELECT,
    USR_OP_DISPLAY_INFO,
    USR_OP_HELP,
    USR_OP_PAGE_UP,
    USR_OP_PAGE_DOWN,
    USR_OP_POWER = 0x40,
    USR_OP_VOLUME_UP,
    USR_OP_VOLUME_DOWN,
    USR_OP_MUTE,
    USR_OP_PLAY,
    USR_OP_STOP,
    USR_OP_PAUSE,
    USR_OP_RECORD,
    USR_OP_REWIND,
    USR_OP_FAST_FORWARD,
    USR_OP_EJECT,
    USR_OP_FORWARD,
    USR_OP_BACKWARD,
    USR_OP_ANGLE = 0x50,
    USR_OP_SUB_PICTURE,
    USR_OP_PLAY_FUNCTION = 0x60,
    USR_OP_PAUSE_PLAY_FUNCTION,
    USR_OP_RECORD_FUNCTION,
    USR_OP_PAUSE_RECORD_FUNCTION,
    USR_OP_STOP_FUNCTION,
    USR_OP_MUTE_FUNCTION,
    USR_OP_RESTORE_VOLUME_FUNCTION,
    USR_OP_TUNE_FUNCTION,
    USR_OP_SELECT_DISK_FUNCTION,
    USR_OP_SELECT_AV_INPUT_FUNCTION,
    USR_OP_SELECT_AU_INPUT_FUNCTION,
    USR_OP_F1_BLUE = 0x71,
    USR_OP_F2_RED,
    USR_OP_F3_GREEN,
    USR_OP_F4_YELLOW,
    USR_OP_F5,
} CEC_USR_CTRL_OP_T;

/**
 * @brief   CEC device types enumeration
 */
typedef enum
{
    CEC_TV,
    CEC_RECORDING1,
    CEC_RECORDING2,
    CEC_TUNER1,
    CEC_PLAYBACK1,
    CEC_AUDIO_SYSTEM,
    CEC_TUNER2,
    CEC_TUNER3,
    CEC_PLAYBACK2,
    CEC_RECORDING3,
    CEC_TUNER4,
    CEC_PLAYBACK3,
    CEC_RESERVED1,
    CEC_RESERVED2,
    CEC_FREE_USE,
    CEC_UNREGISTERED,
} CEC_DEVICE_T;

/**
 * @brief   CEC error enumeration
 */
typedef enum
{
    CEC_RECEIVE_OVERRUN = 1,
    CEC_RECEIVE_BIT_RISING,
    CEC_RECEIVE_SHORT_BIT_PERIOD,
    CEC_RECEIVE_LONG_BIT_PERIOD,
    CEC_RECEIVE_ACKNOWLEDGE,
    CEC_ARB_LOST,
    CEC_TRANSMIT_UNDERRUN,
    CEC_TRANSMIT_ERROR,
    CEC_TRANSMIT_BUSY,
    CEC_TRANSMIT_ACKNOWLEDGE,
    CEC_DEVICE_UNREG,
    CEC_TIMEOUT,
    CEC_OK,
} CEC_ERROR_T;

/**@} end of group APM32F072_EVAL_Enumerates*/

/** @defgroup APM32F072_EVAL_Structures Structures
  @{
  */

/**
 * @brief   CEC device map
 */
typedef struct
{
    uint8_t phyAddr_1;
    uint8_t phyAddr_2;
    uint8_t phyAddr_3;
    uint8_t phyAddr_4;
    uint8_t logicalAddr;
    uint8_t deviceType;
} CEC_MAP_T;

/**
 * @brief   CEC message initialization
 */
typedef struct
{
    __IO uint8_t header;
    __IO uint8_t opcode;
    __IO uint8_t operand[CEC_TX_MSG_LEN_MAX];
    __IO uint8_t txMsgLen;
    __IO uint8_t rxMsgLen;
} CEC_MSG_PACK_T;

/**
 * @brief   CEC message transmission
 */
typedef struct
{
    uint8_t txFrame;
    uint8_t rxFrame;
    uint32_t txCnt;
    uint32_t rxCnt;
    uint8_t txStatus;
    uint8_t rxStatus;
    CEC_ERROR_T transErr;
    uint32_t buffCnt;
    uint8_t buffer[CEC_TX_MSG_LEN_MAX + 1];
} CEC_MSG_TRANS_T;

/**
 * @brief   CEC information
 */
typedef struct
{
    uint16_t physicalAddr;
    uint8_t logicalAddr;
    uint8_t initiatorAddr;
    uint8_t destinationAddr;
    uint8_t deviceType;
    uint16_t deviceConnectSta;
    uint8_t deviceNumber;
} CEC_INFO_T;

/**@} end of group APM32F072_EVAL_Structures*/

/** @defgroup APM32F072_EVAL_Functions Functions
  @{
  */
CEC_ERROR_T CEC_Init(uint8_t initiatorAddr);

#endif

/**@} end of group APM32F072_EVAL_Functions */
/**@} end of group Board_APM32F072_EVAL */
/**@} end of group Board */
